/* eslint-disable */
// Copyright (c) 2016-2018 SmileOnMyMac, LLC dba Smile. All rights reserved

var textexpander = function(te) {
  // I was going to call this te.platform but that's the
  // name of a snippet's abbreviation, so nope.

  te.client = function(client) {
    client.openTemplateUrl = function(url) {
      console.log("open url, replacing parameters with values");
      console.log("platform");
      console.log("osVersion");
      console.log("appName");
      console.log("appVersion");
      console.log("bundleVersion");
    };

    client.setMenu = function(menu) {
      console.log("displaying menu");
      // supports two levels only (bad UI for more anyway)
      // {
      // "&File": [[ commadn1, command2 ] [ command3, command4]],
      // "&Help": [[command1]]
      // }
      // each top level menu is an array or arrays, signifying groups
      // so there will be a seperator between them as needed.
    };

    // menus is an object with named lists of menus, which contain an
    // array of groups, with a group being an array of commands or names of submenus.
    // E.g.
    // setNotificationAreaMenu("start", {
    // "start": [[ command, "nested" ]],
    // "nested": [[ command2 ]]
    //});
    client.setNotificationAreaMenu = function(name, menus) {
      console.log("setting notification area menu");
    };

    // menus is an object with named lists of menus, which contain an
    // array of groups, with a group being an array of commands or names of submenus.
    // E.g.
    // showContextMenu("start", {
    // "start": [[ command, "nested" ]],
    // "nested": [[ command2 ]]
    //});
    client.showContextMenu = function(name, menus) {
      console.log("showing context menu at the current cursor location");
      // menu is : [[command, command2], [ command 3, command 4]]
      // between the groups are a seperator.
    };

    return client;
  }(te.client || {});

  te.user = function(user) {
    var currentUser = { showDemoBanner: false, canExpandByServer: true };

    user.current = function() {
      return currentUser;
    };

    var listeners = [];
    user.addChangeListener = function(listener) {
      listeners.push(listener);
    };

    user.removeChangeListener = function(listener) {
      var index = listeners.indexOf(listener);
      if(index === -1) {
        return;
      }
      var last = listeners.pop();
      if(index !== listeners.length) {
        listeners[index] = last;
      }
    };

    // Only called by the client.
    user.changeUser = function(newUser) {
      currentUser = newUser;

      for(var index = 0; index < listeners.length; ++index) {
        listeners[index](currentUser);
      }
    };

    return user;
  }(te.user || {});

  te.authorization = function(authorization) {
    var lastUserGrant = {'canCreateGroups': false};
    authorization.checkUser = function() {
      return lastUserGrant;
    };

    var userGrantListeners = [];
    authorization.addListenerForUserGrantChanged = function(listener) {
      userGrantListeners.push(listener);
    };
    authorization.removeListenerForUserGrantChanged = function(listener) {
      var index = userGrantListeners.indexOf(listener);
      if(index === -1) {
        return;
      }
      var last = userGrantListeners.pop();
      if(index !== userGrantListeners.length) {
        userGrantListeners[index] = last;
      }
    };

    authorization.raiseUserGrantChanged = function(grant) {
      lastUserGrant = grant;
      for(var index = 0; index < userGrantListeners.length; ++index) {
        userGrantListeners[index](grant);
      }
    }

    return authorization;
  }(te.authorization || {});

  te.commands = function(commands){
    // no commands should be added or removed at runtime.
    // (Apart from the initial initialization in JS)
    commands.all = {};

    var listeners = [];

    commands.notifyChange = function() {
        for(var i = 0; i < listeners.length; ++i){
            listeners[i](window.textexpander.commands.all);
        }
    };

    commands.addChangeListener = function(listener) {
      listeners.push(listener);
    };
    commands.removeChangeListener = function(listener) {
      const index = listeners.indexOf(listener);
      if (index >= 0) {
        listeners.splice(index, 1);
      }
    };

    return commands;
  }(te.commands || {});

  te.snippetList = function(snippetList){
    snippetList.ViewDelegate = function() {};
    snippetList.ViewDelegate.prototype = {
      changeSelectedItem: function(itemId) {
        console.log("selected item changed to " + itemId);
      },
      setCategoryItems: function(categoryItems) {
        console.log("category items in list changed to: " + categoryItems);
      },
      setGroupItems: function(categoryIndex, groupItems) {
        console.log("group items in list changed to: " + groupItems + " for category at index " + categoryIndex);
      },
      setSnippetItems: function (categoryIndex, groupIndex, snippetItems) {
        console.log("snippet items in list changed to: " + snippetItems + " for group at index " + groupIndex);
      },
      setSnippetItem: function (categoryIndex, groupIndex, snippetIndex, snippet) {
        console.log("individual snippet changed, category index: " + categoryIndex +
          ",  group index: " + groupIndex +
          ", snippet index:" + snippetIndex + " with object: " + JSON.stringify(snippet));
      },
      setMatches: function(matches) {
        // N.B. provides the entire set of matches.. e.g.
        // [
        // { snippetId: 1, abbreviation: [ matchdetails ], label: [ matchdetails ], content: [ matchdetails ] },
        // { snippetId: 2, abbreviation: [ matchdetails ], label: [ matchdetails ], content: [ matchdetails ] }
        // ]
        // where the match details are like:
        // { endIndex: 5, match: true } (and match: false if that segment isn't match)
        console.log("Got the following matches: " + JSON.stringify(matches));
      },
      setResultCount: function(countOfFilteredResults) {
        // N.B. null for not applicable.
        console.log("count of filtered results");
      },
      initialLoadCompleted: function() {
        console.log("Initial load of snippet list contents is complete");
      },
      allowSelection: function(allow) {
        console.log("allow selection of items:" + allow);
      }
    };

    var listeners = [];
    snippetList.addViewDelegateChangedListener = function(listener) {
      listeners.push(listener);
    };

    var view = new snippetList.ViewDelegate();
    snippetList.changeViewDelegate = function(newViewDelegate) {
      view = newViewDelegate;
      for(var index = 0; index < listeners.length; ++index) {
        listeners[index](view);
      }
    };

    snippetList.currentViewDelegate = function() {
      return view;
    }

    snippetList.ControllerDelegate = function() {};
    snippetList.ControllerDelegate.prototype = {
      selectedItemChanged: function(itemId) {
        console.log("The selected item has been changed: " + itemId);
      },
      groupIsFoldedChanged: function(groupId, isExpanded) {
        console.log("group folded state is changed: " + groupId + " " + isExpanded);
        // it is expected that view.setSnippetItems will be called as a result
        // of this callback (i.e. it may happen asynchronously)
        // (the group passed in should also have its expanded state updated)
      },
      moveSnippetToGroup: function(snippetId, destinationGroupId) {
        console.log("move snippet: " + snippetId + " to group: " + destinationGroupId);
      },
      newSnippetGroup : function() {
        console.log("creating a new group, returning the id");
        return 12;
      },
      filter : function(text) {
        console.log("restricting view to snippets containing" + text);
        return;
      }
    };

    if(!snippetList.controller) {
        snippetList.controller = new snippetList.ControllerDelegate();
    }

    return snippetList;
  }(te.snippetList || {});

  te.groupDetails = function(groupDetails){
    groupDetails.ViewDelegate = function() {};
    groupDetails.ViewDelegate.prototype = {
      setGroupDetails: function(groupDetails) {
        console.log("Changed group details");
      },
      getGroupDetails: function() {
        console.log("Client requested group details");
        /* share button types (match the group state), the correct label will be set
        0 - Personal
        1 - Shared
        2 - Organisation
        3 - Public Subscription */
        return {"name" : "Group name goes here",
                "notes" : "Group notes go here",
                "expandAfter" : 0,
                "shareButtonType" : 0,
                "isEditable" : true,
                "prefix" : '1st',
                "canOverride" : true,
                "overriddenPrefix" : "2nd",
                "isPrefixEditable": true}
      }
    };

    var listeners = [];
    groupDetails.addViewDelegateChangedListener = function(listener) {
      listeners.push(listener);
    };

    var view = new groupDetails.ViewDelegate();
    groupDetails.changeViewDelegate = function(newViewDelegate) {
      view = newViewDelegate;
      for(var index = 0; index < listeners.length; ++index) {
        listeners[index](view);
      }
    };

    groupDetails.currentViewDelegate = function() {
      return view;
    }

    groupDetails.ControllerDelegate = function() {};
    groupDetails.ControllerDelegate.prototype = {
      setGroupDetails: function(groupDetails) {
        console.log("The changed group contents!");
      },
      groupHasUnsavedChanges: function() {
        console.log("Group has unsaved changes");
      },
      groupChangesHaveBecomeIdle: function() {
        console.log("Group changes have become idle");
      },
      clickedShareButton: function() {
        console.log("Selected group share button");
      }
    };

    if(!groupDetails.controller) {
        groupDetails.controller = new groupDetails.ControllerDelegate();
    }

    return groupDetails;
  }(te.groupDetails || {});

  return te;
}(textexpander || {});
